import json

from tensorflow.keras.applications import Xception,VGG16,VGG19,ResNet50,ResNet101,ResNet152,ResNet50V2,ResNet101V2
from tensorflow.keras.applications import ResNet152V2,InceptionV3,MobileNet,MobileNetV2,DenseNet121,DenseNet169
from tensorflow.keras.applications import DenseNet201,NASNetMobile,EfficientNetB0,EfficientNetB1,EfficientNetB2
from tensorflow.keras.applications.inception_resnet_v2 import InceptionResNetV2
from tensorflow.keras.applications.efficientnet import EfficientNetB3, EfficientNetB4, EfficientNetB5, \
    EfficientNetB6, EfficientNetB7
from tensorflow.keras.applications.nasnet import NASNetLarge
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException
import sys

class modellconfig:
    def __init__(self,model_name):
        self.model_name = model_name['MODEL_OBJ']
        self.logger = App_Logger()

    def configureData(self,params):
            SIZE = params['IMAGE_SIZE'].split(',')
            h = int(SIZE[0])
            w = int(SIZE[1])
            c = int(SIZE[2])
            IMAGE_SIZE = h, w, c
            CONFIG = {
                'TRAIN_DATA_DIR' : params['TRAIN_DATA_DIR'],
                'VALID_DATA_DIR' : params['VALID_DATA_DIR'],
                'AUGMENTATION': params['AUGMENTATION'],
                'CLASSES' : params['CLASSES'],
                'IMAGE_SIZE' : IMAGE_SIZE,
                'BATCH_SIZE' : params['BATCH_SIZE'],
            }

            return CONFIG

    def return_model(self):
        try:
            with open("Config_Layer/configs.json", 'r') as f:
                self.params = json.load(f)

            data_config = self.configureData(self.params)

            if self.model_name == 'Xception':
                print("Loading Xception..")
                core_model = Xception(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'VGG16':
                print("Loading VGG16..")
                core_model = VGG16(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'VGG19':
                print("Loading VGG19..")
                core_model = VGG19(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'ResNet50':
                print("Loading ResNet50..")
                core_model = ResNet50(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'ResNet101':
                print("Loading ResNet101..")
                core_model = ResNet101(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'ResNet152':
                print("Loading ResNet152..")
                core_model = ResNet152(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'ResNet50V2':
                print("Loading ResNet50V2..")
                core_model = ResNet50V2(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'ResNet101V2':
                print("Loading ResNet101V2..")
                core_model = ResNet101V2(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'ResNet152V2':
                print("Loading ResNet152V2..")
                core_model = ResNet152V2(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'InceptionV3':
                print("Loading InceptionV3..")
                core_model = InceptionV3(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'InceptionResNetV2':
                print("Loading InceptionResNetV2..")
                core_model = InceptionResNetV2(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])


            elif self.model_name == 'MobileNet':
                print("Loading MobileNet..")
                core_model = MobileNet(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'MobileNetV2':
                print("Loading MobileNetV2..")
                core_model = MobileNetV2(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'DenseNet121':
                print("Loading DenseNet121..")
                core_model = DenseNet121(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'DenseNet169':
                print("Loading DenseNet169..")
                core_model = DenseNet169(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'DenseNet201':
                print("Loading DenseNet201..")
                core_model = DenseNet201(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'NASNetMobile':
                print("Loading NASNetMobile..")
                core_model = NASNetMobile(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'NASNetLarge':
                print("Loading NASNetLarge..")
                core_model = NASNetLarge(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB0':
                print("Loading EfficientNetB0..")
                core_model = EfficientNetB0(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB1':
                print("Loading EfficientNetB1..")
                core_model = EfficientNetB1(include_top=False,weights="imagenet",input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB2':
                print("Loading EfficientNetB2..")
                core_model = EfficientNetB2(include_top=False, weights="imagenet", input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB3':
                print("Loading EfficientNetB3..")
                core_model = EfficientNetB3(include_top=False, weights="imagenet", input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB4':
                print("Loading EfficientNetB4..")
                core_model = EfficientNetB4(include_top=False, weights="imagenet", input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB5':
                print("Loading EfficientNetB5..")
                core_model = EfficientNetB5(include_top=False, weights="imagenet", input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB6':
                print("Loading EfficientNetB6..")
                core_model = EfficientNetB6(include_top=False, weights="imagenet", input_shape=data_config['IMAGE_SIZE'])

            elif self.model_name == 'EfficientNetB7':
                print("Loading EfficientNetB7..")
                core_model = EfficientNetB7(include_top=False, weights="imagenet", input_shape=data_config['IMAGE_SIZE'])

            return core_model
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modellconfig.__name__,
                         self.return_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)




from tensorflow.keras.applications import Xception,VGG16,VGG19,ResNet50,ResNet101,ResNet152,ResNet50V2,ResNet101V2
from tensorflow.keras.applications import ResNet152V2,InceptionV3,MobileNet,MobileNetV2,DenseNet121,DenseNet169
from tensorflow.keras.applications import DenseNet201,NASNetMobile,EfficientNetB0,EfficientNetB1,EfficientNetB2
from tensorflow.keras.applications.inception_resnet_v2 import InceptionResNetV2
from tensorflow.keras.applications.efficientnet import EfficientNetB3, EfficientNetB4, EfficientNetB5, \
    EfficientNetB6, EfficientNetB7
from tensorflow.keras.applications.nasnet import NASNetLarge
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException
import sys

class modellconfig:
    def __init__(self,model_name):
        self.x = model_name['MODEL_OBJ']
        self.logger = App_Logger()

    def return_model(self,param,img_iteration=1,batch_iteration=1,opt_iteration=1):
        try:
            self.params = param
            data_config = self.params
            sizes = data_config['IMAGE_SIZE'].split(',')
            size = [int(j) for j in sizes]
            temp = len(size)-1
            ch = size[temp]
            core_model = []
            if isinstance(self.x, list):
                print("list")
            else:
                self.x = [self.x]
            for i in range(img_iteration):
                for j in range(batch_iteration):
                    for k in range(opt_iteration):
                        for model_name in self.x:
                            print(model_name)
                            if model_name == 'Xception':
                                print("Loading Xception..")
                                core_model.append(Xception(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'VGG16':
                                print("Loading VGG16..")
                                core_model.append(VGG16(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'VGG19':
                                print("Loading VGG19..")
                                core_model.append(VGG19(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'ResNet50':
                                print("Loading ResNet50..")
                                core_model.append(ResNet50(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'ResNet101':
                                print("Loading ResNet101..")
                                core_model.append(ResNet101(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'ResNet152':
                                print("Loading ResNet152..")
                                core_model.append(ResNet152(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'ResNet50V2':
                                print("Loading ResNet50V2..")
                                core_model.append(ResNet50V2(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'ResNet101V2':
                                print("Loading ResNet101V2..")
                                core_model.append(ResNet101V2(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'ResNet152V2':
                                print("Loading ResNet152V2..")
                                core_model.append(ResNet152V2(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'InceptionV3':
                                print("Loading InceptionV3..")
                                core_model.append(InceptionV3(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'InceptionResNetV2':
                                print("Loading InceptionResNetV2..")
                                core_model.append(InceptionResNetV2(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))


                            elif model_name == 'MobileNet':
                                print("Loading MobileNet..")
                                core_model.append(MobileNet(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'MobileNetV2':
                                print("Loading MobileNetV2..")
                                core_model.append(MobileNetV2(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'DenseNet121':
                                print("Loading DenseNet121..")
                                core_model.append(DenseNet121(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'DenseNet169':
                                print("Loading DenseNet169..")
                                core_model.append(DenseNet169(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'DenseNet201':
                                print("Loading DenseNet201..")
                                core_model.append(DenseNet201(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'NASNetMobile':
                                print("Loading NASNetMobile..")
                                core_model.append(NASNetMobile(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'NASNetLarge':
                                print("Loading NASNetLarge..")
                                core_model.append(NASNetLarge(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB0':
                                print("Loading EfficientNetB0..")
                                core_model.append(EfficientNetB0(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB1':
                                print("Loading EfficientNetB1..")
                                core_model.append(EfficientNetB1(include_top=False,weights="imagenet",input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB2':
                                print("Loading EfficientNetB2..")
                                core_model.append(EfficientNetB2(include_top=False, weights="imagenet", input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB3':
                                print("Loading EfficientNetB3..")
                                core_model.append(EfficientNetB3(include_top=False, weights="imagenet", input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB4':
                                print("Loading EfficientNetB4..")
                                core_model.append(EfficientNetB4(include_top=False, weights="imagenet", input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB5':
                                print("Loading EfficientNetB5..")
                                core_model.append(EfficientNetB5(include_top=False, weights="imagenet", input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB6':
                                print("Loading EfficientNetB6..")
                                core_model.append(EfficientNetB6(include_top=False, weights="imagenet", input_shape=(size[i],size[i],ch)))

                            elif model_name == 'EfficientNetB7':
                                print("Loading EfficientNetB7..")
                                core_model.append(EfficientNetB7(include_top=False, weights="imagenet", input_shape=(size[i],size[i],ch)))

            return core_model
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modellconfig.__name__,
                         self.return_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)




import json

from cv_image_suit.utils.experiment_utils.exp_config import config
import tensorflow as tf
from tensorflow.keras import models
from cv_image_suit.utils.experiment_utils import exp_models_config
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException
import sys

class modell:
    def __init__(self):
        self.confige = config("Config_Layer/experiment_inputs_configs.json")
        self.logger = App_Logger()


    def get_model(self,phase,img_iteration=1,batch_iteration=1,opt_iteration=1):
        """The logic for loading pretrain model.

         Args:
          MODEL_OBJ: Model object

        Returns:
          It returns keras model objcet

        """
        try:

            self.param = self.confige.load_data()
            if phase == "IMAGE_SIZE":
                size = self.param['IMAGE_SIZE'].split(',')
                img_iteration = len(size) - 1
                with open("Config_Layer/experiment_result.json", "r") as f:
                    jason_param = json.load(f)
                    f.close()
                self.mc = exp_models_config.modellconfig(jason_param)
            elif phase == "BATCH_SIZE":
                with open("Config_Layer/experiment_result.json", "r") as f:
                    jason_param = json.load(f)
                    f.close()
                self.mc = exp_models_config.modellconfig(jason_param)
            elif phase == "OPTIMIZER":
                with open("Config_Layer/experiment_result.json", "r") as f:
                    jason_param = json.load(f)
                    f.close()
                self.mc = exp_models_config.modellconfig(jason_param)
            else:
                self.mc = exp_models_config.modellconfig(self.param)
            self.config_model = self.confige.configureModel(self.param)
            self.model = self.mc.return_model(self.param,img_iteration,batch_iteration,opt_iteration)
            self.config_data = self.confige.configureData(self.param)
            model = self.model
            print("Detected pretrain model!!")
            return model


        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, modell.__name__,
                            self.get_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()

            raise Exception(exception_msg)


    def model_preparation(self,model):
        try:
            final_models = []
            i=0
            opt=self.config_model['OPTIMIZER']
            if isinstance(opt, list):
                print("list")
            else:
                opt = [opt]
            for x in model:
                print('Preparing model...')
                if self.config_model['FREEZE_ALL'] == 'True':
                    print('Freezing all...')
                    for layer in x.layers:
                        layer.trainable = False

                    # Add custom layers
                    flatten_layer = tf.keras.layers.Flatten()

                    if self.config_data['CLASSES'] > 2:
                        print('Adding softmax...')
                        prediction = tf.keras.layers.Dense(
                            units=self.config_data['CLASSES'],
                            activation="softmax"
                        )#(flatten_in)

                        full_model = models.Sequential([
                                        x,
                                        flatten_layer,
                                        prediction
                                    ])

                        full_model.compile(
                            optimizer=opt[i],
                            loss="sparse_categorical_crossentropy",
                            metrics=["accuracy"]
                        )
                        print('Model loaded!!')

                        final_models.append(full_model)
                        i=i+1
                    else:
                        print('Adding sigmoid...')
                        prediction = tf.keras.layers.Dense(
                            units=1,
                            activation="sigmoid"
                        )#(flatten_in)

                        full_model = models.Sequential([
                                        x,
                                        flatten_layer,
                                        prediction
                                    ])

                        full_model.compile(
                            optimizer=opt[i],
                            loss="binary_crossentropy",
                            metrics=["accuracy"]
                        )

                        print('Model loaded!!')

                        final_models.append(full_model)
                        i=i+1

                else:

                    for layer in x.layers[:self.config_model['FREEZE_TILL']]:
                        layer.trainable = False
                    for layer in x.layers[self.config_model['FREEZE_TILL']:]:
                        layer.trainable = True

                        # Add custom layers
                    flatten_layer = tf.keras.layers.Flatten()#(model.output)

                    if self.config_data['CLASSES'] > 2:
                        prediction = tf.keras.layers.Dense(
                            units=self.config_data['CLASSES'],
                            activation="softmax"
                        )#(flatten_in)

                        full_model = models.Sequential([
                            x,
                            flatten_layer,
                            prediction
                        ])

                        full_model.compile(
                            optimizer=opt[i],
                            loss="sparse_categorical_crossentropy",
                            metrics=["accuracy"]
                        )
                        print('Model loaded!!')

                        final_models.append(full_model)
                        i=i+1

                    else:
                        prediction = tf.keras.layers.Dense(
                            units=1,
                            activation="sigmoid"
                        )#(flatten_in)

                        full_model = models.Sequential([
                            x,
                            flatten_layer,
                            prediction
                        ])

                        full_model.compile(
                            optimizer=opt[i],
                            loss="binary_crossentropy",
                            metrics=["accuracy"]
                        )

                        print('Model loaded!!')

                        final_models.append(full_model)
                        i=i+1
            return final_models
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, modell.__name__,
                            self.get_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()

            raise Exception(exception_msg)


    def load_pretrain_model(self,phase,img_iteration=1,batch_iteration=1,opt_iteration=1):

        """The logic for loading pretrain model.

         Args:
          MODEL_OBJ: Model object

        Returns:
          It returns keras model objcet

        """
        try:
            model = self.get_model(phase,img_iteration,batch_iteration,opt_iteration)
            models = self.model_preparation(model)
            return models
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, modell.__name__,
                            self.get_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)















import base64
import numpy
from os import listdir

def decodeImage(imgstring, fileName):
    imgdata = base64.b64decode(imgstring)
    with open(fileName, 'wb') as f:
        f.write(imgdata)
        f.close()


def encodeImageIntoBase64(croppedImagePath):
    with open(croppedImagePath, "rb") as f:
        return base64.b64encode(f.read())


def batch_validate(train_dir,valid_dir,percent):
    train_dir = train_dir
    val_dir = valid_dir

    number_of_samples=[]
    for i in listdir(train_dir):
        j = len(listdir(train_dir+'/'+i))
        number_of_samples.append(j)

    for i in listdir(val_dir):
        j = len(listdir(val_dir+'/'+i))
        number_of_samples.append(j)

    minimum_samples = numpy.min(number_of_samples)
    print("min samples ---------------")
    print(number_of_samples)
    print(minimum_samples)
    res = float(minimum_samples) * float(percent)
    print(res)
    batch = numpy.math.ceil(res/100)
    return batch


import json

class config:
    """Public cv_image_suit utilities.
    This module is used as a shortcut to access all the symbols. Those symbols was
    exposed under train engine and predict engine.
    """
    def __init__(self,filename):
        self.filename =filename

    def load_data(self):
        with open(self.filename,'r') as f:
              params = json.load(f)
        return params

    def configureData(self,params):
        SIZE = params['IMAGE_SIZE'].split(',')
        BATCH = params['BATCH_SIZE']
        CONFIG = {
            'TRAIN_DATA_DIR' : params['TRAIN_DATA_DIR'],
            'VALID_DATA_DIR' : params['VALID_DATA_DIR'],
            'AUGMENTATION': params['AUGMENTATION'],
            'CLASSES' : params['CLASSES'],
            'IMAGE_SIZE' : SIZE,
            'BATCH_SIZE' : BATCH,
            'PERCENT_DATA' : params['PERCENT_DATA']
        }

        return CONFIG


    def configureModel(self, params):
        opt = params['OPTIMIZER']
        CONFIG = {
            'EXP_NAME' : params['EXP_NAME'],
            'EPOCHS' : params['EPOCHS'],
            'OPTIMIZER' : opt,
            'FREEZE_ALL' : params['FREEZE_ALL'],
            'FREEZE_TILL' : params['FREEZE_TILL']
        }

        return CONFIG


from cv_image_suit.utils.normal_utils import models_config
import json

class config:
    """Public cv_image_suit utilities.
    This module is used as a shortcut to access all the symbols. Those symbols was
    exposed under train engine and predict engine.
    """
    def __init__(self,filename):
        self.filename =filename

    def load_data(self):
        with open(self.filename,'r') as f:
              params = json.load(f)
        return params

    def load_pred_data(self):
        with open("Config_Layer/pred_configs.json",'r') as f:
              params = json.load(f)
        return params

    def configureData(self,params):
        SIZE = params['IMAGE_SIZE'].split(',')
        h = int(SIZE[0])
        w = int(SIZE[1])
        c = int(SIZE[2])
        IMAGE_SIZE = h, w, c
        CONFIG = {
            'TRAIN_DATA_DIR' : params['TRAIN_DATA_DIR'],
            'VALID_DATA_DIR' : params['VALID_DATA_DIR'],
            'AUGMENTATION': params['AUGMENTATION'],
            'CLASSES' : params['CLASSES'],
            'IMAGE_SIZE' : IMAGE_SIZE,
            'BATCH_SIZE' : params['BATCH_SIZE'],
        }

        return CONFIG


    def configureModel(self, params):
        self.pram = self.load_data()
        self.mc = models_config.modellconfig(self.pram)
        CONFIG = {
            'MODEL_OBJ' : self.mc.return_model(),
            'MODEL_NAME' : params['MODEL_NAME'],
            'EPOCHS' : params['EPOCHS'],
            'OPTIMIZER' : params['OPTIMIZER'],
            'FREEZE_ALL' : params['FREEZE_ALL'],
            'FREEZE_TILL' : params['FREEZE_TILL'],
            'RESUME' : params['RESUME']
        }

        return CONFIG


import json
import sys

import numpy
import tensorflow as tf

from cv_image_suit.Exception_Layer.exception import GenericException
from cv_image_suit.utils.experiment_utils.exp_config import config
from cv_image_suit.utils.experiment_utils import exp_model, exp_callbacks, exp_data_manager
from cv_image_suit.Logging_Layer.logger import App_Logger

class modelfinder:
    def __init__(self,filename):
        self.filename = filename
        self.config = config(self.filename)
        self.call = exp_callbacks.callback()
        self.mod = exp_model.modell()
        self.dm = exp_data_manager.datamanage()
        self.logger = App_Logger()

    def find_model(self,phase,model_iteration=1):
        try:
            self.phase = phase
            param = self.config.load_data()

            final_data = self.config.configureData(param)
            batch_iteration = 0
            model_names = param['MODEL_OBJ']
            if not isinstance(model_names,list):
                model_names = [model_names]

            final_model_data = self.config.configureModel(param)
            self.log_dir = self.call.get_log_path()

            log_dir = self.log_dir
            tb_cb = tf.keras.callbacks.TensorBoard(log_dir=log_dir)
            call = [tb_cb]

            if self.phase == "BATCH_SIZE":
                temp = final_data['BATCH_SIZE']
                batch_iteration = len(temp)
                train_data, valid_data = self.dm.train_valid_generator(batch_iteration=batch_iteration)
                with open("Config_Layer/experiment_result.json", "r") as f:
                    json_obj = json.load(f)
                    f.close()
                model_names = json_obj["MODEL_OBJ"]
                model_names = [model_names]
                z = len(model_names) - 1
                batch_iteration = len(train_data)
                model_obj = self.mod.load_pretrain_model(self.phase,batch_iteration=batch_iteration)
                my_model = model_obj
                acc = self.find_batchsize(my_model, model_names, z, final_data, batch_iteration, final_model_data,
                                          train_data, valid_data, call)
                best_model = self.write_model_result(acc)
                return best_model
            elif self.phase == "IMAGE_SIZE":
                temp = final_data['IMAGE_SIZE']
                img_iteration = len(temp)-1
                train_data, valid_data = self.dm.train_valid_generator(img_iteration)
                with open("Config_Layer/experiment_result.json", "r") as f:
                    json_obj = json.load(f)
                    f.close()
                model_names=json_obj["MODEL_OBJ"]
                model_names=[model_names]
                z = len(model_names)-1
                model_obj = self.mod.load_pretrain_model(self.phase,img_iteration=img_iteration)
                my_model = model_obj
                acc = self.find_imagesize(my_model, model_names, z, phase, final_data, img_iteration, final_model_data,
                                          train_data, valid_data, call)
                best_model = self.write_model_result(acc)
                return best_model
            elif self.phase == "OPTIMIZER":
                train_data, valid_data = self.dm.train_valid_generator()
                with open("Config_Layer/experiment_result.json", "r") as f:
                    json_obj = json.load(f)
                    f.close()
                opt = final_model_data['OPTIMIZER']
                opt_iteration = len(opt)
                model_names=json_obj["MODEL_OBJ"]
                model_names=[model_names]
                z = len(model_names)-1
                model_obj = self.mod.load_pretrain_model(self.phase,opt_iteration=opt_iteration)
                my_model = model_obj
                acc = self.optimizer_find(my_model, model_names,z,opt,opt_iteration, final_model_data, train_data, valid_data, call)
                best_model = self.write_model_result(acc)
                return best_model
            else:
                img_iteration=1
                train_data, valid_data = self.dm.train_valid_generator(img_iteration)
                z = len(model_names) - 1
                model_obj = self.mod.load_pretrain_model(self.phase)
                my_model = model_obj
                acc = self.model_find(my_model, model_names, final_data, final_model_data, train_data, valid_data, call)
                best_model = self.write_model_result(acc)
                return best_model
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modelfinder.__name__,
                         self.find_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)


    def model_find(self,my_model,model_names,final_data,final_model_data,train_data,valid_data,call):
        try:
            acc = {}
            i = 0
            image_size = final_data["IMAGE_SIZE"]
            temp = len(image_size)-1
            h = str(image_size[0])
            w = str(image_size[0])
            ch = str(image_size[temp])
            image_size = str(h+','+w+','+ch)

            steps_per_epoch = (train_data[0].samples // valid_data[0].batch_size)
            validation_steps = (train_data[0].samples // valid_data[0].batch_size)
            print(my_model)
            for x in my_model:
                history = x.fit(
                    train_data[0],
                    validation_data=valid_data[0],
                    epochs=final_model_data['EPOCHS'],
                    steps_per_epoch=steps_per_epoch,
                    validation_steps=validation_steps,
                    callbacks=call
                )
                val_acc = history.history['val_accuracy']
                res=0
                for val in val_acc:
                    res = res+val
                res = res/len(val_acc)
                acc[res]=[model_names[i], image_size, valid_data[0].batch_size,res]
                i = i + 1
            return acc
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modelfinder.__name__,
                         self.model_find.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


    def write_model_result(self,acc,opt = "Adam"):
        try:
            temp = []
            for res in acc.keys():
                temp.append(res)
            max_acc = numpy.max(temp)
            best_model = acc[max_acc]
            if self.phase == "OPTIMIZER":
                CONFIG = {"MODEL_OBJ": best_model[0],
                          "IMAGE_SIZE": best_model[1],
                          "BATCH_SIZE": best_model[2],
                          "VAL_ACC": best_model[3],
                          "OPTIMIZER": best_model[4]
                          }
            else:
                CONFIG = {"MODEL_OBJ": best_model[0],
                          "IMAGE_SIZE": best_model[1],
                          "BATCH_SIZE": best_model[2],
                          "VAL_ACC": best_model[3],
                          "OPTIMIZER": opt
                          }

            with open("Config_Layer/experiment_result.json", "w") as f:
                json.dump(CONFIG, f)
            return CONFIG
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modelfinder.__name__,
                         self.write_model_result.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


    def fetch_img_size(self):
        with open("Config_Layer/experiment_result.json", "r") as f:
            jason_obj = json.load(f)
            img_size = jason_obj["IMAGE_SIZE"]
            f.close()
        return img_size

    def fetch_batch(self):
        with open("Config_Layer/experiment_result.json", "r") as f:
            jason_obj = json.load(f)
            batch_size = jason_obj["BATCH_SIZE"]
            f.close()
        return batch_size

    def optimizer_find(self,my_model,model_names,z,opt,opt_iteration,final_model_data,train_data,valid_data,call):
        try:
            acc = {}
            image_size = self.fetch_img_size()
            batch = self.fetch_batch()
            steps_per_epoch = (train_data[0].samples // valid_data[0].batch_size)
            validation_steps = (train_data[0].samples // valid_data[0].batch_size)

            for k in range(opt_iteration):
                history = my_model[k].fit(
                    train_data[0],
                    validation_data=valid_data[0],
                    epochs=final_model_data['EPOCHS'],
                    steps_per_epoch=steps_per_epoch,
                    validation_steps=validation_steps,
                    callbacks=call
                )
                val_acc = history.history['val_accuracy']
                res=0
                for val in val_acc:
                    res = res+val
                res = res/len(val_acc)
                acc[res]=[model_names[z], str(image_size),batch,res,opt[k]]
            return acc
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modelfinder.__name__,
                         self.optimizer_find.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


    def find_batchsize(self,my_model,model_names,z,final_data,batch_iteration,final_model_data,train_data,valid_data,call):
        try:
            acc = {}
            batch_size = final_data["BATCH_SIZE"]
            image_size = self.fetch_img_size()
            for k in range(len(train_data)):
                steps_per_epoch = (train_data[k].samples // valid_data[k].batch_size)
                validation_steps = (train_data[k].samples // valid_data[k].batch_size)

                history = my_model[k].fit(
                    train_data[k],
                    validation_data=valid_data[k],
                    epochs=final_model_data['EPOCHS'],
                    steps_per_epoch=steps_per_epoch,
                    validation_steps=validation_steps,
                    callbacks=call
                )
                val_acc = history.history['val_accuracy']
                res=0
                for val in val_acc:
                    res = res+val
                res = res/len(val_acc)
                acc[res]=[model_names[z], str(image_size), batch_size[k],res]
            return acc
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modelfinder.__name__,
                         self.find_batchsize.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


    def find_imagesize(self,my_model,model_names,z,phase,final_data,img_iteration,final_model_data,train_data,valid_data,call):
        try:
            acc = {}
            i = 0
            j = 0
            image_size = final_data["IMAGE_SIZE"]
            for k in range(img_iteration):

                steps_per_epoch = (train_data[k].samples // valid_data[k].batch_size)
                validation_steps = (train_data[k].samples // valid_data[k].batch_size)

                n = len(my_model)
                if i < n:
                    history = my_model[i].fit(
                        train_data[j],
                        validation_data=valid_data[j],
                        epochs=final_model_data['EPOCHS'],
                        steps_per_epoch=steps_per_epoch,
                        validation_steps=validation_steps,
                        callbacks=call
                    )
                    val_acc = history.history['val_accuracy']
                    res=0
                    for val in val_acc:
                        res = res+val
                    res = res/len(val_acc)
                    ch = len(image_size)-1
                    acc[res]=[model_names[z], str(image_size[k])+','+str(image_size[k])+','+str(image_size[ch]), valid_data[k].batch_size,res]
                    i = i + 1
                j = j + 1
            return acc
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, modelfinder.__name__,
                         self.find_imagesize.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()














from cv_image_suit.utils.normal_utils.config import config
from tensorflow.keras.preprocessing.image import ImageDataGenerator
from tensorflow.keras.preprocessing import image
import numpy as np
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException
import sys

class datamanage:
    def __init__(self):
        self.confige = config("Config_Layer/configs.json")
        self.logger = App_Logger()

    def train_valid_generator(self):

        '''
        This function generates train & valid data from images path,
        it also performs data augmentation.
        :return object of data:
        '''

        try:
            class_mode = ""
            self.param = self.confige.load_data()
            self.config_data = self.confige.configureData(self.param)
            if self.config_data['CLASSES'] > 2:
                class_mode = "sparse"
            else:
                class_mode = "binary"
            if self.config_data['AUGMENTATION'] == 'True':
                print("Augmetation applied!")
                train_datagen = ImageDataGenerator(rescale=1. / 255,
                                                   shear_range=0.2,
                                                   zoom_range=0.2,
                                                   horizontal_flip=True)

                valid_datagen = ImageDataGenerator(rescale=1. / 255)

                training_set = train_datagen.flow_from_directory(
                    directory= self.config_data['TRAIN_DATA_DIR'],
                    target_size=self.config_data['IMAGE_SIZE'][:-1],
                    batch_size=self.config_data['BATCH_SIZE'],
                    class_mode=class_mode)

                valid_set = valid_datagen.flow_from_directory(
                    directory=self.config_data['VALID_DATA_DIR'],
                    target_size=self.config_data['IMAGE_SIZE'][:-1],
                    batch_size=self.config_data['BATCH_SIZE'],
                    class_mode=class_mode)

                return training_set, valid_set

            else:
                train_datagen = ImageDataGenerator(rescale=1. / 255)
                valid_datagen = ImageDataGenerator(rescale=1. / 255)

                training_set = train_datagen.flow_from_directory(
                    directory=self.config_data['TRAIN_DATA_DIR'],
                    target_size=self.config_data['IMAGE_SIZE'][:-1],
                    batch_size=self.config_data['BATCH_SIZE'],
                    class_mode=class_mode)

                valid_set = valid_datagen.flow_from_directory(
                    directory=self.config_data['VALID_DATA_DIR'],
                    target_size=self.config_data['IMAGE_SIZE'][:-1],
                    batch_size=self.config_data['BATCH_SIZE'],
                    class_mode=class_mode)

                return training_set, valid_set
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, datamanage.__name__,
                         self.train_valid_generator.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


    def class_name(self):
        try:
            train, valid = self.train_valid_generator()
            return train.class_indices
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, datamanage.__name__,
                         self.train_valid_generator.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


    def manage_input_data(self,INPUT_IMAGE):
        '''
        This function takes an new raw image & convert the image
        into nd array with respect to training images dimention.
        :param INPUT_IMAGE:
        :return nd array:
        '''
        try:
            self.param = self.confige.load_data()
            self.config_data = self.confige.configureData(self.param)
            INPUT_IMG = INPUT_IMAGE
            test_image = image.load_img(INPUT_IMG, target_size= self.config_data['IMAGE_SIZE'][:-1])
            test_image = image.img_to_array(test_image)
            test_image = np.expand_dims(test_image, axis=0)

            return test_image
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, datamanage.__name__,
                            self.train_valid_generator.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()


import os
import time
import tensorflow as tf
from cv_image_suit.utils.experiment_utils.exp_config import config

class callback:
    """Configures callbacks for use in various training loops.
      Args:
          get_log_path: unique log location.
          checkpoint : None
      Returns:
          Instance of CallbackList used to control all Callbacks.
     """
    def __init__(self):
        self.config = config("Config_Layer/experiment_inputs_configs.json")


    def get_log_path(self):
        param = self.config.load_data()
        final_data = self.config.configureModel(param)
        self.log_dir = "Tensorboard/logs/" + final_data['EXP_NAME']
        fileName = time.strftime("log_%Y_%m_%d_%H_%M_%S")
        logs_path = os.path.join(self.log_dir, fileName)
        print(f"Saving logs at {logs_path}")
        return logs_path





import sys
from os import listdir
import numpy as np
import pandas
from tensorflow.keras.models import load_model

from cv_image_suit.com_in_ineuron_ai_utils.utils import decodeImage, encodeImageIntoBase64
from cv_image_suit.utils.normal_utils import data_manager
from cv_image_suit.utils.normal_utils.config import config
from cv_image_suit.Exception_Layer.exception import GenericException
from cv_image_suit.Logging_Layer.logger import App_Logger
class tfpredict:
    def __init__(self,filename, pred_dir=None):
        self.filename =filename
        self.confige = config("Config_Layer/configs.json")
        self.dm = data_manager.datamanage()
        self.logger = App_Logger()
        self.pred_dir = pred_dir
    def predictiontf(self):
        try:
            self.param = self.confige.load_data()
            self.config_model = self.confige.load_pred_data()
            self.data_config = self.confige.configureData(self.param)

            # load model
            model_path = f"New_trained_model/{self.config_model['MODEL_NAME']}"
            print('Loading...', model_path)
            model = load_model(model_path)

            # summarize model
            #model.summary()

            names = self.dm.class_name()
            class_names = list(names.keys())
            if self.config_model['PRED_TYPE'] == 'Single Prediction':
                imagename = self.filename
                predict = self.dm.manage_input_data(imagename)
                if self.data_config['CLASSES'] > 2:
                    result = model.predict(predict/255)
                    results = np.argmax(result)
                    return [{ "image_class" : str(class_names[results])}]
                else:
                    result = model.predict(predict/255)
                    predicted_classes = [1 * (x[0] >= 0.5) for x in result]
                    return [{ "image_class" : str(class_names[predicted_classes[0]])}]
            else:
                predictions=[]
                img_names=[]
                for x in listdir(self.pred_dir):
                    x_encode = encodeImageIntoBase64(self.pred_dir+'/'+x)
                    decodeImage(x_encode, self.filename)
                    predict = self.dm.manage_input_data(self.filename)
                    if self.data_config['CLASSES'] > 2:
                        result = model.predict(predict/255)
                        results = np.argmax(result)
                        predictions.append(str(class_names[results]))
                        img_names.append(str(x))

                        #return [{ "image_class" : str(class_names[results])}]
                    else:
                        result = model.predict(predict/255)
                        predicted_classes = [1 * (x[0] >= 0.5) for x in result]
                        predictions.append(str(class_names[predicted_classes[0]]))
                        img_names.append(str(x))
                        #return [{ "image_class" : str(class_names[predicted_classes[0]])}]
                result_dataframe = pandas.DataFrame(list(zip(img_names,predictions)), columns=['Image', 'Predictions'])
                path = "Prediction_Output_File/"
                result_dataframe.to_csv(path + "Predictions.csv", header=True, mode='w')
                return path + "Predictions.csv"
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, tfpredict.__name__,
                         self.predictiontf.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)








import os

class GenericException(Exception):

    def __init__(self, error_message):
        """
        :param error_message: error_message in string format
        """
        self.error_message = error_message

    def __repr__(self):
        return GenericException.__name__.__str__()

    def error_message_detail(self, error, error_detail):
        """
        This function raise error message
        :param error(__str__): your error message
        :param error_detail(__str__):error detail message
        :return:__type__:error_message
        """

        exec_type, exc_obj, exc_tb = error_detail.exc_info()
        file_name = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        error_message = "Python script name [{0}] line number [{1}] error message [{2}]".format(file_name, exc_tb.tb_lineno, str(error))

        self.error_message = self.error_message + " " + error_message
        return self.error_message
    def __str__(self):
        return self.error_message


from setuptools import find_packages, setup

setup(
  name = 'cv_image_suit',
  packages = find_packages(),
  include_package_data=True,
  version = '0.9',
  license='GNU',
  description = 'Its an auto image classification and experimentation library',
  long_description='cv_image_suit is a deep learning image classification library written in Python, running on top of the machine learning platform TensorFlow.Keras. It was developed with a focus on enabling fast experimentation of images classification. You can classify any image with any classification model in Keras appliaction without writing any lines of code.',
  long_description_content_type="text/markdown",
  url="https://github.com/uday446/cv_image_suit",
  author = 'Udayrajsinh Jadeja',
  author_email = 'ujadeja96@gmail.com',
  keywords = ['cv_image_suit'],
  install_requires=[
        'tensorflow==2.9.1',
        'scipy',
        'numpy',
        'pandas',
        'Pillow==8.3.2',
        'fastapi',
  ],
  classifiers=[
    'Development Status :: 3 - Alpha',

    # Indicate who your project is intended for
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',

    # Pick your license as you wish
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
  entry_points={
        "console_scripts": [
            "cv_image_suit = cv_image_suit.main:start_app",
        ]},
)



from cv_image_suit.com_in_ineuron_ai_utils.utils import batch_validate
from cv_image_suit.utils.experiment_utils.exp_config import config
from tensorflow.keras.preprocessing.image import ImageDataGenerator
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException
import sys

class datamanage:
    def __init__(self):
        self.confige = config("Config_Layer/experiment_inputs_configs.json")
        self.logger = App_Logger()

    def train_valid_generator(self,iteration=1,batch_iteration=1):

        '''
        This function generates train & valid data from images path,
        it also performs data augmentation.
        :return object of data:
        '''

        try:
            training_set=[]
            valid_set=[]
            class_mode = ""
            self.param = self.confige.load_data()
            self.config_data = self.confige.configureData(self.param)
            percentage = 100.0 - float(self.config_data['PERCENT_DATA'])
            percent = float(percentage / 100)
            sizes = self.config_data['IMAGE_SIZE']
            size = [int(j) for j in sizes]
            batch_size = self.config_data['BATCH_SIZE']

            print("batch Info-------------------")
            print(batch_iteration)
            #print(allowed_batch)

            allowed_batch = batch_validate(self.config_data['TRAIN_DATA_DIR'], self.config_data['VALID_DATA_DIR'], self.config_data['PERCENT_DATA'])

            for i in range(iteration):
                for j in range(batch_iteration):
                    if batch_size [j] > allowed_batch:
                        break
                    if self.config_data['CLASSES'] > 2:
                        class_mode = "sparse"
                    else:
                        class_mode = "binary"
                    if self.config_data['AUGMENTATION'] == 'True':
                        print("Augmetation applied!")
                        train_datagen = ImageDataGenerator(rescale=1. / 255,
                                                           shear_range=0.2,
                                                           zoom_range=0.2,
                                                           horizontal_flip=True,
                                                           validation_split=percent)

                        valid_datagen = ImageDataGenerator(rescale=1. / 255,
                                                           validation_split=percent)

                        train_set = train_datagen.flow_from_directory(
                            directory= self.config_data['TRAIN_DATA_DIR'],
                            target_size=(size[i],size[i]),
                            batch_size=int(batch_size[j]),
                            class_mode=class_mode,
                            subset='training')

                        val_set = valid_datagen.flow_from_directory(
                            directory=self.config_data['VALID_DATA_DIR'],
                            target_size=(size[i],size[i]),
                            batch_size=int(batch_size[j]),
                            class_mode=class_mode,
                            subset='training')
                        training_set.append(train_set)
                        valid_set.append(val_set)
                        #return training_set, valid_set

                    else:
                        train_datagen = ImageDataGenerator(rescale=1. / 255,validation_split=percent)
                        valid_datagen = ImageDataGenerator(rescale=1. / 255,validation_split=percent)

                        train_set = train_datagen.flow_from_directory(
                            directory=self.config_data['TRAIN_DATA_DIR'],
                            target_size=(size[i],size[i]),
                            batch_size=int(batch_size[j]),
                            class_mode=class_mode,
                            subset='training')

                        val_set = valid_datagen.flow_from_directory(
                            directory=self.config_data['VALID_DATA_DIR'],
                            target_size=(size[i],size[i]),
                            batch_size=int(batch_size[j]),
                            class_mode=class_mode,
                            subset='training')
                        training_set.append(train_set)
                        valid_set.append(val_set)
                        #return training_set, valid_set
            return training_set, valid_set
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, datamanage.__name__,
                            self.train_valid_generator.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()











import sys

from cv_image_suit.utils.normal_utils.config import config
from cv_image_suit.utils.normal_utils import callbacks, data_manager, model
import tensorflow as tf
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException


class tftrainer:
    def __init__(self):
        self.confige = config("Config_Layer/configs.json")
        self.call = callbacks.callback()
        self.mod = model.modell()
        self.dm = data_manager.datamanage()
        self.logger = App_Logger()

    def train(self):

        """The logic for one training step.

        This method should contain the mathematical logic for one step of training.
        This typically includes the forward pass, loss calculation, backpropagation,
        and metric updates.

         Args:
          data: A nested structure of `Tensor`s.

        Returns:
          A `dict` containing values.Typically, the
          values of the `Model`'s metrics are returned. Example:
          `{'loss': 0.2, 'accuracy': 0.7}`.

        """
        try:
            self.param = self.confige.load_data()
            self.config_model = self.confige.configureModel(self.param)
            self.config_data = self.confige.configureData(self.param)
            self.log_dir = self.call.get_log_path()
            self.ckp = self.call.checkpoint()
            model_obj = self.mod.load_pretrain_model()
            my_model = model_obj

            #my_model.load_weights("cv_image_suit/Checkpoint/Model_ckpt.h5")

            train_data, valid_data = self.dm.train_valid_generator()

            #callbacks
            log_dir = self.log_dir
            tb_cb = tf.keras.callbacks.TensorBoard(log_dir=log_dir)
            model_checkpoint = self.ckp[0]
            early_stopping = self.ckp[1]
            lr_reducer = self.ckp[2]

            call = [tb_cb, model_checkpoint, early_stopping, lr_reducer]

            #Calculating steps_per_epoch & validation_steps
            steps_per_epoch = train_data.samples // train_data.batch_size
            validation_steps = valid_data.samples // valid_data.batch_size

            my_model.fit(
                train_data,
                validation_data=valid_data,
                epochs=self.config_model['EPOCHS'],
                steps_per_epoch=steps_per_epoch,
                validation_steps=validation_steps,
                callbacks=call
            )

            new_path = f"New_trained_model/{'new'+self.config_model['MODEL_NAME']+'.h5'}"
            my_model.save(new_path)
            return f"Model saved at the following location : {new_path}"
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(self.__module__, tftrainer.__name__,
                         self.train.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)




import os
import time
import tensorflow as tf

class callback:
    """Configures callbacks for use in various training loops.
      Args:
          get_log_path: unique log location.
          checkpoint : None
      Returns:
          Instance of CallbackList used to control all Callbacks.
     """
    def __init__(self):
        self.log_dir="Tensorboard/logs/fit"

    def get_log_path(self):
      fileName = time.strftime("log_%Y_%m_%d_%H_%M_%S")
      logs_path = os.path.join(self.log_dir, fileName)
      print(f"Saving logs at {logs_path}")
      return logs_path


    def checkpoint(self):
        CKPT_path = "Checkpoint/Model_ckpt.h5"
        checkpointing_cb = tf.keras.callbacks.ModelCheckpoint(CKPT_path, save_best_only=True)
        earlystoping = tf.keras.callbacks.EarlyStopping('val_loss',patience=8,verbose=1,mode='min')
        reducer = tf.keras.callbacks.ReduceLROnPlateau('val_loss',patience=5,verbose=1,mode='min',min_lr=0.00001)
        return  [checkpointing_cb,earlystoping,reducer]




from cv_image_suit.utils.normal_utils.config import config
import tensorflow as tf
from tensorflow.keras import models
import os
from cv_image_suit.Logging_Layer.logger import App_Logger
from cv_image_suit.Exception_Layer.exception import GenericException
import sys

class modell:
    def __init__(self):
        self.confige = config("Config_Layer/configs.json")
        self.logger = App_Logger()

    def get_model(self):
        """The logic for loading pretrain model.

         Args:
          MODEL_OBJ: Model object

        Returns:
          It returns keras model objcet

        """
        try:
            self.param = self.confige.load_data()
            self.config_model = self.confige.configureModel(self.param)
            self.config_data = self.confige.configureData(self.param)
            model =  self.config_model['MODEL_OBJ']
            print("Detected pretrain model!!")
            return model

        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, modell.__name__,
                            self.get_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)


    def model_preparation(self,model):
        try:
            print('Preparing model...')
            if self.config_model['RESUME'] == "True":
                model = tf.keras.models.load_model(os.getcwd()+"/Checkpoint/Model_ckpt.h5")
                return model
            else:
                if self.config_model['FREEZE_ALL'] == 'True':
                    print('Freezing all...')
                    for layer in model.layers:
                        layer.trainable = False

                    # Add custom layers
                    flatten_layer = tf.keras.layers.Flatten()

                    if self.config_data['CLASSES'] > 2:
                        print('Adding softmax...')
                        prediction = tf.keras.layers.Dense(
                            units=self.config_data['CLASSES'],
                            activation="softmax"
                        )  # (flatten_in)

                        full_model = models.Sequential([
                            model,
                            flatten_layer,
                            prediction
                        ])

                        full_model.compile(
                            optimizer=self.config_model['OPTIMIZER'],
                            loss="sparse_categorical_crossentropy",
                            metrics=["accuracy"]
                        )
                        print('Model loaded!!')

                        return full_model

                    else:
                        print('Adding sigmoid...')
                        prediction = tf.keras.layers.Dense(
                            units=1,
                            activation="sigmoid"
                        )  # (flatten_in)

                        full_model = models.Sequential([
                            model,
                            flatten_layer,
                            prediction
                        ])

                        full_model.compile(
                            optimizer=self.config_model['OPTIMIZER'],
                            loss="binary_crossentropy",
                            metrics=["accuracy"]
                        )

                        print('Model loaded!!')

                        return full_model

                else:

                    for layer in model.layers[:self.config_model['FREEZE_TILL']]:
                        layer.trainable = False
                    for layer in model.layers[self.config_model['FREEZE_TILL']:]:
                        layer.trainable = True

                    # Add custom layers
                    flatten_layer = tf.keras.layers.Flatten()  # (model.output)

                    if self.config_data['CLASSES'] > 2:
                        prediction = tf.keras.layers.Dense(
                            units=self.config_data['CLASSES'],
                            activation="softmax"
                        )  # (flatten_in)

                        full_model = models.Sequential([
                            model,
                            flatten_layer,
                            prediction
                        ])

                        full_model.compile(
                            optimizer=self.config_model['OPTIMIZER'],
                            loss="sparse_categorical_crossentropy",
                            metrics=["accuracy"]
                        )
                        print('Model loaded!!')

                        return full_model

                    else:
                        prediction = tf.keras.layers.Dense(
                            units=1,
                            activation="sigmoid"
                        )  # (flatten_in)

                        full_model = models.Sequential([
                            model,
                            flatten_layer,
                            prediction
                        ])

                        full_model.compile(
                            optimizer=self.config_model['OPTIMIZER'],
                            loss="binary_crossentropy",
                            metrics=["accuracy"]
                        )

                        print('Model loaded!!')

                        return full_model
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, modell.__name__,
                            self.get_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)

    def load_pretrain_model(self):

        """The logic for loading pretrain model.

         Args:
          MODEL_OBJ: Model object

        Returns:
          It returns keras model objcet

        """
        try:
            model = self.get_model()
            model = self.model_preparation(model)
            return model
        except Exception as e:
            exception = GenericException(
                "Failed during training in module [{0}] class [{1}] method[{2}]"
                    .format(self.__module__, modell.__name__,
                            self.get_model.__name__))
            file = open("Logs/logs.txt", 'a+')
            exception_msg = exception.error_message_detail(str(e), sys)
            self.logger.log(file, "Exception occured %s " % exception_msg)
            file.close()
            raise Exception(exception_msg)














import sys
from cv_image_suit.Logging_Layer.logger import App_Logger
import numpy
from cv_image_suit.com_in_ineuron_ai_utils.utils import decodeImage
from cv_image_suit.Prediction_Layer.predict import tfpredict
import json
import webbrowser
from threading import Timer
from cv_image_suit.Training_Layer.train_engine import tftrainer
from cv_image_suit.Experiment_Layer.model_finder import modelfinder
from os import listdir
import os
from tensorboard import program
from cv_image_suit.com_in_ineuron_ai_utils.utils import batch_validate
from cv_image_suit.Exception_Layer.exception import GenericException
from fastapi import FastAPI, Request, Form
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from fastapi.responses import FileResponse
from fastapi.middleware.cors import CORSMiddleware
import uvicorn
from fastapi.encoders import jsonable_encoder

class ClientApp:
    def __init__(self,pred_dir=None):
        self.filename = "inputImage.jpg"
        self.pred_dir = pred_dir
        self.classifier = tfpredict(self.filename, self.pred_dir)
        self.tftraining = tftrainer()
        self.modelfinder = modelfinder("Config_Layer/experiment_inputs_configs.json")
        self.logger = App_Logger()



app = FastAPI() # initializing a flask app
origins = ["*"]
app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["POST"],
    allow_headers=["*"],
    max_age=2
    )
path = [p for p in sys.path if p.endswith('site-packages')]
print(path[0])
os.chdir(str(path[0])+"/cv_image_suit")
app.mount("/static", StaticFiles(directory="static"), name="static")
templates = Jinja2Templates(directory="templates")


@app.get("/", response_class=HTMLResponse) # route to display the home page
async def homePage(request:Request):
    return templates.TemplateResponse("index.html", {"request": request})

@app.get('/input2', response_class=HTMLResponse)  # route to display the home page
async def input_form2(request:Request):
    return templates.TemplateResponse("input_experiments.html", {"request": request})

@app.get('/input3',response_class=HTMLResponse)  # route to display the home page
async def input_form3(request:Request):
    exps=[]
    if os.path.isdir(os.getcwd() + "/Tensorboard/logs/"):
        for x in listdir(os.getcwd() + "/Tensorboard/logs/"):
            exps.append(x)
    if not isinstance(exps, list):
        exps = [exps]
    return templates.TemplateResponse("experiment_log.html", {"request": request, "exps":exps})

@app.get('/input',response_class=HTMLResponse)  # route to display the home page
def input_form(request:Request):
    model_list = []
    if os.path.isdir(os.getcwd()+"/New_trained_model"):
        for x in listdir(os.getcwd()+"/New_trained_model"):
            model_list.append(x)
    return templates.TemplateResponse("input_form.html", {"request": request, "model_list": model_list})

@app.get('/download',)
async def downloadFile():
    file_name = "experiment_result.json"
    file_path = os.getcwd() + "/Config_Layer/" + file_name
    return FileResponse(path=file_path, media_type='application/octet-stream', filename=file_name)

@app.get('/download2')
async def downloadFile2():
    file_name = "Predictions.csv"
    file_path = os.getcwd() + "/Prediction_Output_File/" + file_name
    return FileResponse(path=file_path, media_type='application/octet-stream', filename=file_name)

@app.post('/experiment',response_class=HTMLResponse)
async def experiment_func(request:Request,MODEL_OBJ: list = Form(...),OPTIMIZER: list = Form(...)):

    clApp = ClientApp()
    try:
        form_data = await request.form()
        # Data config
        TRAIN_DATA_DIR = form_data['TRAIN_DATA_DIR']
        VALID_DATA_DIR = form_data['VALID_DATA_DIR']
        CLASSES = int(form_data['CLASSES'])
        IMAGE_SIZE = form_data['IMAGE_SIZE']
        AUGMENTATION = form_data['AUGMENTATION']
        BATCH_SIZE = form_data['BATCH_SIZE']
        PERCENT_DATA = int(form_data['PERCENTAGE_DATA'])

        BATCH = BATCH_SIZE.split(',')
        BATCH = [int(j) for j in BATCH]
        BATCH_SIZE = numpy.sort(BATCH)
        BATCH_SIZE = [int(j) for j in BATCH_SIZE]
        size = IMAGE_SIZE.split(',')
        size = [int(j) for j in size]
        if PERCENT_DATA > 100:
            return templates.TemplateResponse("input_experiments.html",
                                              {"request": request,
                                               "error": ["PERCENT_DATA cannot be more than 100"]}
                                              )
        size = numpy.sort(size)
        size = [int(j) for j in size]
        allowed_batch = batch_validate(TRAIN_DATA_DIR,VALID_DATA_DIR,PERCENT_DATA)
        if BATCH_SIZE[0] > allowed_batch:
            return templates.TemplateResponse("input_experiments.html",
                                              {"request": request,
                                               "error": ["Image size must be larger than 71x71 and last value must 3"]}
                                              )
        elif size[1] < 71 and size[0] != 3:
            return templates.TemplateResponse("input_experiments.html",
                                              {"request": request,
                                               "error": ["Image size must be larger than 71x71 and last value must 3"]}
                                              )

        MODEL_OBJ = MODEL_OBJ
        EXP_NAME = form_data['EXP_NAME']
        FREEZE_ALL = form_data['FREEZE_ALL']
        FREEZE_TILL = int(form_data['FREEZE_TILL'])
        OPTIMIZER = OPTIMIZER
        EPOCHS = int(form_data['EPOCHS'])

        configs = {
            "TRAIN_DATA_DIR": TRAIN_DATA_DIR,
            "VALID_DATA_DIR": VALID_DATA_DIR,
            "AUGMENTATION": AUGMENTATION,
            "CLASSES": CLASSES,
            "IMAGE_SIZE": IMAGE_SIZE,
            "BATCH_SIZE": BATCH_SIZE,
            "MODEL_OBJ": MODEL_OBJ,
            "EXP_NAME": EXP_NAME,
            "EPOCHS": EPOCHS,
            "FREEZE_ALL": FREEZE_ALL,
            "FREEZE_TILL": FREEZE_TILL,
            "OPTIMIZER": OPTIMIZER,
            "PERCENT_DATA": PERCENT_DATA
        }

        with open("Config_Layer/experiment_inputs_configs.json", "w") as f:
            json.dump(configs, f)
            f.close()

        Phase = "MODEL"
        best_model = clApp.modelfinder.find_model(phase=Phase)

        Phase = "IMAGE_SIZE"
        best_model = clApp.modelfinder.find_model(phase=Phase)

        Phase = "BATCH_SIZE"
        best_model = clApp.modelfinder.find_model(phase=Phase)

        Phase = "OPTIMIZER"
        best_model = clApp.modelfinder.find_model(phase=Phase)

        exps = []
        if os.path.isdir(os.getcwd() + "/Tensorboard/logs/"):
            for x in listdir(os.getcwd() + "/Tensorboard/logs/"):
                exps.append(x)
        if not isinstance(exps,list):
            exps = [exps]
        return templates.TemplateResponse("experiment_output.html",
                                          {"request": request,
                                           "model_list": best_model,
                                           "exps":exps}
                                          )
    except Exception as e:
        exception = GenericException(
            "Failed during training in module [{0}] class [{1}] method[{2}]"
                .format(ClientApp.__module__, ClientApp.__name__,
                        train_func.__name__))
        file = open("Logs/logs.txt", 'a+')
        exception_msg = exception.error_message_detail(str(e), sys)
        clApp.logger.log(file, "Exception occured %s " % exception_msg)
        file.close()
        raise Exception(exception_msg)


@app.post('/train',response_class=HTMLResponse) # route to show the predictions in a web UI
async def train_func(request:Request):

    clApp = ClientApp()
    try:
        form_data = await request.form()
        # Data config
        TRAIN_DATA_DIR = form_data['TRAIN_DATA_DIR']
        VALID_DATA_DIR = form_data['VALID_DATA_DIR']
        CLASSES =int(form_data['CLASSES'])
        IMAGE_SIZE = form_data['IMAGE_SIZE']
        AUGMENTATION = form_data['AUGMENTATION']
        BATCH_SIZE = int(form_data['BATCH_SIZE'])


        # Model config
        MODEL_OBJ = form_data['MODEL_OBJ']
        MODEL_NAME = form_data['MODEL_NAME']
        FREEZE_ALL = form_data['FREEZE_ALL']
        FREEZE_TILL = form_data['FREEZE_TILL']
        OPTIMIZER = form_data['OPTIMIZER']
        EPOCHS = int(form_data['EPOCHS'])
        RESUME = form_data['RESUME']

        size = IMAGE_SIZE.split(',')
        size = [int(j) for j in size]
        size = numpy.sort(size)
        size = [int(j) for j in size]
        PERCENT_DATA=100
        allowed_batch = batch_validate(TRAIN_DATA_DIR,VALID_DATA_DIR,PERCENT_DATA)
        if BATCH_SIZE > allowed_batch:
            return templates.TemplateResponse("input_form.html",
                                              {"request": request ,
                                               "error": ["Please provide smaller batch size"]})
        elif size[1] < 71 and size[0] != 3:
            return templates.TemplateResponse("input_form.html",
                                              {"request": request,
                                               "error": ["Image size must be larger than 71x71 and last value must 3"]})

        configs = {
            "TRAIN_DATA_DIR": TRAIN_DATA_DIR,
            "VALID_DATA_DIR": VALID_DATA_DIR,
            "AUGMENTATION": AUGMENTATION,
            "CLASSES": CLASSES,
            "IMAGE_SIZE": IMAGE_SIZE,
            "BATCH_SIZE": BATCH_SIZE,
            "MODEL_OBJ": MODEL_OBJ,
            "MODEL_NAME": MODEL_NAME,
            "EPOCHS": EPOCHS,
            "FREEZE_ALL": FREEZE_ALL,
            "FREEZE_TILL": FREEZE_TILL,
            "OPTIMIZER": OPTIMIZER,
            "RESUME": RESUME
        }

        with open("Config_Layer/configs.json", "w") as f:
            json.dump(configs, f)
            f.close()

        hist = clApp.tftraining.train()
        my_list = ["Traning Completed  ",hist]
        return templates.TemplateResponse("input_form.html",
                                          {"request": request,
                                           "output": my_list})
    except Exception as e:
        exception = GenericException(
            "Failed during training in module [{0}] class [{1}] method[{2}]"
            .format(ClientApp.__module__, ClientApp.__name__,
                     train_func.__name__))
        file = open("Logs/logs.txt", 'a+')
        exception_msg = exception.error_message_detail(str(e), sys)
        clApp.logger.log(file, "Exception occured %s " % exception_msg)
        file.close()
        raise Exception(exception_msg)

@app.get('/mid',response_class=HTMLResponse)  # route to display the home page
async def pred(request:Request):
    model_list = []
    if os.path.isdir(os.getcwd() + "/New_trained_model"):
        for x in listdir(os.getcwd()+"/New_trained_model"):
            model_list.append(x)
    return templates.TemplateResponse("mid_form.html", {"request": request, "model_list": model_list,"error":""})

@app.post('/test',response_class=HTMLResponse)  # route to display the home page
async def predcit(request:Request):
    form_data = await request.form()
    model_list = []
    if os.path.isdir(os.getcwd() + "/New_trained_model"):
        for x in listdir(os.getcwd() + "/New_trained_model"):
            model_list.append(x)
    model_found=0
    for x in model_list:
        if form_data['MODEL_NAME'] == x:
            model_found = 1
        else:
            model_found = 0
    if model_found == 0:
        return templates.TemplateResponse("mid_form.html", {"request": request, "model_list": model_list, "error": "Please Provide Model From Above List"})
    configs = {
        "MODEL_NAME": form_data['MODEL_NAME'],
        "PRED_TYPE": form_data['PRED_TYPE']
    }

    with open("Config_Layer/pred_configs.json", "w") as f:
        json.dump(configs, f)
        f.close()
    if form_data['PRED_TYPE'] == "Single Prediction":
        return templates.TemplateResponse("predict.html", {"request": request})
    else:
        return templates.TemplateResponse("predict_all.html", {"request": request})

@app.get('/logs',response_class=HTMLResponse)  # route to display the home page
async def log(request:Request):
    tracking_address = "Tensorboard/logs/fit"
    tb = program.TensorBoard()
    tb.configure(argv=[None, '--logdir', tracking_address])
    url = tb.launch()
    print(f"Tensorflow listening on {url}")
    webbrowser.open_new(url)
    return templates.TemplateResponse("input_form.html", {"request": request})

@app.post('/explogs',response_class=HTMLResponse)  # route to display the home page
async def log2(request:Request,exp: str = Form(...)):
    try:
        tracking_address = "Tensorboard/logs/"+exp
        tb = program.TensorBoard()
        tb.configure(argv=[None, '--logdir', tracking_address])
        url = tb.launch()
        webbrowser.open_new(url)
        print(f"Tensorflow listening on {url}")
        return templates.TemplateResponse("experiment_log.html", {"request": request,"url":url})
    except Exception as e:
        print(str(e))
        return templates.TemplateResponse("experiment_log.html", {"request": request})

@app.post("/predict")
async def predictRoute(request:Request):
    clApp = ClientApp()
    img = await request.json()
    image = img['image']
    decodeImage(image, clApp.filename)
    result = clApp.classifier.predictiontf()
    return jsonable_encoder(result)

@app.post("/predict_all",response_class=HTMLResponse)
async def predictAllRoute(request:Request, PRED : str = Form(...)):
    clApp = ClientApp(PRED)
    result = clApp.classifier.predictiontf()
    return templates.TemplateResponse("predict_all.html", {"request": request, "error":["Prediction File Generated!!"]})

@app.get("/view_logs",response_class=HTMLResponse)
async def viewlogs(request:Request):
    with open('Logs/logs.txt','r') as f:
        lines = f.readlines()
        f.close()
    return templates.TemplateResponse("show_logs.html",
                                      {"request": request, "logs": lines})

def open_browser():
    webbrowser.open_new('http://127.0.0.1:8080/')


def start_app():
    Timer(3, open_browser).start()
    port = 8080
    uvicorn.run(app, host="0.0.0.0", port=port)



if __name__ == "__main__":
    start_app()
